/**
  *  \file planetscentral/dbclient/connection.h
  *  \brief Database Client: Generic Connection Handling
  */
#ifndef C2SERVER_PLANETSCENTRAL_DBCLIENT_CONNECTION_H
#define C2SERVER_PLANETSCENTRAL_DBCLIENT_CONNECTION_H

#include <vector>
#include "cpluslib/string.h"
#include "cpluslib/types.h"
#include "int/data.h"

typedef std::vector<int32_t> IntList_t;
typedef std::vector<string_t> StringList_t;

/** Connection. Provides buffered access to a socket or pipe. Together
    with that, provides functions to serialize and deserialize objects
    in "redis" protocol format.

    The file descriptors can be obtained in three ways:
    - using connect()/connectRetry()/maybeReconnect() for outbound
      network connections
    - using accept() for inbound network connections
    - using use() with externally provided fds, for example, from a pipe

    Buffered access is required to achieve reasonable performance. */
class Connection {
 public:
    Connection();
    virtual ~Connection();

    // socket management
    void connect(string_t host, string_t port);
    void connectRetry(string_t host, string_t port);
    bool maybeReconnect(string_t host, string_t port);
    bool accept(int socketfd);
    void use(int infd, int outfd);
    void close();
    void checkConnected();
    bool isConnected() const;

    // buffered I/O
    IntValue* readObject(bool acceptShortForm);
    void      sendObject(IntValue* v);
    void      sendArray(const IntDataSegment& seg, uint32_t size);
    void      sendError(string_t err);
    void      sendSuccess(string_t msg);
    void      flush();
    bool      isAtEnd();

    void      send(const char* p, std::size_t n);
    char      readChar();
    void      readLine(string_t& line);

 protected:
    virtual void onConnect();

 private:
    // buffered receive
    void    refillBuffer();
    int32_t readInt();
    void    readBytes(string_t& line, std::size_t size);

    // file descriptor
    int infd;
    int outfd;

    // receive buffer
    char buffer[1024];
    std::size_t buf_fill;
    std::size_t buf_ptr;

    // send buffer
    char txbuffer[1024];
    std::size_t tx_fill;
};

int32_t toIntResult(IntValue* value);
string_t toStringResult(IntValue* value);
void toIntListResult(IntValue* value, IntList_t& values);
void toStringListResult(IntValue* value, StringList_t& values);

IntValue* getArrayElem(IntValue* value, uint32_t idx);
uint32_t  getArraySize(IntValue* value);
IntValue* getHashElem(IntValue* value, string_t name);
IntValue* getHashElem(IntValue* value, const char* name);

#endif
