/**
  *  \file planetscentral/dirinfo.cc
  *  \brief File Server - Directory Game Information
  */

#include "planetscentral/dirinfo.h"
#include "io/dirs.h"
#include "io/streams.h"
#include "cpluslib/misc.h"
#include "cpluslib/format.h"

/** Constructor. Creates the object and initializes it.
    \param dir Directory to read */
DirInfo::DirInfo(Directory& dir)
    : have_race_names(false)
{
    init(dir);
}

DirInfo::~DirInfo()
{ }

/** Check for presence of registration.
    \retval true There is a registration (registered or shareware)
    \retval false There is no registration */
bool
DirInfo::haveRegInfo() const
{
    return !reg_file_name.empty();
}

/** Get registration. If the directory contains no registration,
    this returns a standard, shareware key. */
const GRegInfo&
DirInfo::getRegInfo() const
{
    return reg_info;
}

/** Get name of registration file. If the directory contains no
    registration, this returns an empty string. */
const string_t&
DirInfo::getRegFileName() const
{
    return reg_file_name;
}

/** Check for presence of game data.
    \retval true This directory contains at least one result file
    \retval false This directory contains no result files */
bool
DirInfo::haveGame() const
{
    return (overview.getGlobalFlags() & GDirectoryOverview::have_RST) != 0;
}

/** Get overview data. The GDirectoryOverview object contains
    all available RST files. */
const GDirectoryOverview&
DirInfo::getOverview() const
{
    return overview;
}

/** Check for presence of race names.
    \retval true This directory contains race names
    \retval false This directory contains no race names */
bool
DirInfo::haveRaceNames() const
{
    return have_race_names;
}

/** Get race names. If this directory does not contain a race.nm file,
    returns an empty object. */
const GRaceNameList&
DirInfo::getRaceNames() const
{
    return race_names;
}

/** Get list of missing files. */
const DirInfo::MissingList_t&
DirInfo::getMissingFiles() const
{
    return missing_files;
}

/** Initialize. */
void
DirInfo::init(Directory& dir)
{
    /* Use default race names */
    race_names = host_racenames;
    have_race_names = false;

    try {
        /* Step 1: Load registration */
        if (Ptr<Stream> s = dir.openFileNT("fizz.bin", Stream::C_READ)) {
            reg_info.parseFizz(*s);
            reg_file_name = "fizz.bin";
        }

        /* Step 2: Load overview */
        overview.scan(dir);

        /* Step 3: Load race names */
        if (Ptr<Stream> s = dir.openFileNT("race.nm", Stream::C_READ)) {
            race_names.load(*s);
            have_race_names = true;
        } else {
            missing_files.push_back("race.nm");
        }

        /* Step 4: Missing files. Only check when we have some players! */
        if (haveGame()) {
            static const char*const files[] = {
                "beamspec.dat",
                "engspec.dat",
                "hullspec.dat",
                "pconfig.src",
                "planet.nm",
                "torpspec.dat",
                "truehull.dat",
            };
            for (std::size_t i = 0; i < countof(files); ++i) {
                if (!dir.openFileNT(files[i], Stream::C_READ)) {
                    missing_files.push_back(files[i]);
                }
            }

            if (!dir.openFileNT("xyplan.dat", Stream::C_READ)) {
                bool haveXY = true;
                for (int i = 1; i <= NUM_PLAYERS; ++i) {
                    if (overview.getPlayerFlags(i) & overview.have_RST) {
                        if (!dir.openFileNT(format("xyplan%d.dat", i), Stream::C_READ)) {
                            haveXY = false;
                            break;
                        }
                    }
                }
                if (!haveXY) {
                    missing_files.push_back("xyplan.dat");
                }
            }
        }
    }
    catch (...) {
        /* Ignore all errors. This would mean the directory is not
           recognized as a proper game directory. */
    }
}
