/**
  *  \file planetscentral/talk/cmdfolder.cc
  *  \brief Talk: FOLDER commands
  */

#include <algorithm>
#include "planetscentral/talk/cmdfolder.h"
#include "planetscentral/dbclient/reconnect.h"
#include "planetscentral/talk/talkconnection.h"
#include "planetscentral/talk/user.h"
#include "planetscentral/talk/userfolder.h"
#include "planetscentral/talk/talk.h"
#include "planetscentral/talk/userpm.h"
#include "planetscentral/talk/cmdforum.h"

/** Configure a folder. */
static void
configureFolder(UserFolder& folder, IntArgBlock& args)
{
    while (args.getNumArgs() > 0) {
        args.checkArgumentCountAtLeast(2);
        string_t key = toStringResult(args.getNext());
        string_t val = toStringResult(args.getNext());
        folder.header().setField(key, val);
    }
}

/* @q FOLDERLS (Talk Command)
   List user's PM folders.

   Permissions: user context required, accesses user's folders

   @retval UFID[] list of folder Ids
   @uses default:folder:all, user:$UID:pm:folder:all */
void
doFolderList(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCount(0);

    // Do it
    ReconnectingGuard g(database_rc);
    IntList_t result;
    UserFolder::defaultFolders(database_connection).getAll(result);
    User(database_connection, conn.getUser()).pmFolders().getAll(result);

    // Convert to array and send it
    IntDataSegment ds;
    for (IntList_t::size_type i = 0; i < result.size(); ++i) {
        ds.pushNew(makeIntValue(result[i]));
    }
    conn.sendArray(ds, result.size());
}

/* @q FOLDERSTAT folder:UFID (Talk Command)
   Information about one folder.

   Permissions: user context required, accesses user's folders

   @err 404 Folder does not exist
   @retval TalkFolderInfo
   @uses user:$UID:pm:folder:$UFID:header, default:folder:$UFID:header, user:$UID:pm:folder:$UFID:messages
   @see FOLDERMSTAT */
void
doFolderStat(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCount(1);
    int32_t ufid = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, ufid);
    std::auto_ptr<IntValue> result(folder.describe(folder.checkExistance()));
    conn.sendObject(result.get());
}

/* @q FOLDERMSTAT folder:UFID... (Talk Command)
   Information about multiple folders.

   Permissions: user context required, accesses user's folders

   @retval TalkFolderInfo[] information for all folders, null if a folder does not exist
   @uses user:$UID:pm:folder:$UFID:header, default:folder:$UFID:header, user:$UID:pm:folder:$UFID:messages
   @see FOLDERSTAT */
void
doFolderMStat(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    IntDataSegment result;
    while (args.getNumArgs() > 0) {
        int32_t ufid = toIntResult(args.getNext());
        std::auto_ptr<IntValue> tmp;
        try {
            UserFolder folder(u, ufid);
            tmp.reset(folder.describe(folder.checkExistance()));
        }
        catch (...) { }
        result.pushNew(tmp.release());
    }
    conn.sendArray(result, result.size());
}

/* @q FOLDERNEW name:Str [key:Str value:Str ...] (Talk Command)
   Create new folder.
   The command can include %key/%value pairs that are used to configure the folder, see {FOLDERSET}.

   Permissions: user context required, accesses user's folders

   @retval UFID new folder Id
   @uses user:$UID:pm:folder:id, user:$UID:pm:folder:$UFID:header */
void
doFolderNew(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCountAtLeast(1);
    string_t name = toStringResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());

    // Create the folder
    int32_t newUfid = UserFolder::allocateFolder(u);
    UserFolder folder(u, newUfid);
    u.pmFolders().add(newUfid);

    // Configure the folder
    folder.header().setField("name", name);
    configureFolder(folder, args);

    IntIntIntValue iv(newUfid);
    conn.sendObject(&iv);
}

/* @q FOLDERRM folder:UFID (Talk Command)
   Remove folder.
   Deletes the folder and all messages it contains.

   Permissions: user context required, accesses user's folders

   @retval Int 1=success, 0=folder not deleted because it did not exist or was not deletable
   @uses user:$UID:pm:folder:all, user:$UID:pm:folder:$UFID:messages */
void
doFolderRemove(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCount(1);
    int32_t ufid = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, ufid);

    // Try to remove it.
    if (!u.pmFolders().remove(ufid)) {
        // Remove failed, this is not a user folder. Either it did not exist,
        // or it is a global default folder.
        conn.sendSuccess("0");
    } else {
        // Unlink all messages
        IntList_t msgs;
        folder.messages().getAll(msgs);
        for (IntList_t::iterator i = msgs.begin(); i != msgs.end(); ++i) {
            UserPM(database_connection, *i).removeReference();
        }

        // Remove the folder
        folder.remove();
        conn.sendSuccess("1");
    }
}

/* @q FOLDERSET folder:UFID [key:Str value:Str ...] (Talk Command)
   Configure folder.
   The given %key/%value pairs are set in {user:$UID:pm:folder:$UFID:header}.

   Valid keys:
   - name (folder name)
   - description (description/subtitle)

   Permissions: user context required, accesses user's folders

   @uses user:$UID:pm:folder:$UFID:header */
void
doFolderSet(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    args.checkArgumentCountAtLeast(1);
    int32_t ufid = toIntResult(args.getNext());

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, ufid);
    folder.checkExistance();
    configureFolder(folder, args);

    conn.sendSuccess("OK");
}

/* @q FOLDERLSPM folder:UFID [listParameters...] (Talk Command)
   Query list of PMs in a user's folder.

   The list can be accessed in different ways, see {pcc:talk:listparams|listParameters}.
   Valid sort keys for folders are:
   - author
   - subject
   - time

   Permissions: user context required, accesses user's folders

   @rettype Any
   @uses user:$UID:pm:folder:$UFID:messages */
void
doFolderListPMs(TalkConnection& conn, IntArgBlock& args)
{
    conn.checkUser();
    ListParams parms;
    parms.parse(args, true, ListParams::pmSortKeys);

    // Do it
    ReconnectingGuard g(database_rc);
    User u(database_connection, conn.getUser());
    UserFolder folder(u, parms.getId());
    folder.checkExistance();
    parms.exec(conn, folder.messages());
}
